#include <windows.h>
#include <stdio.h>
#include "blockio.h"

const int serialcommbuffer=maxblocksizeserial+8+2;	// block plus header

HANDLE hSerialPort=INVALID_HANDLE_VALUE;
	//In summary, set both readIntervalTimeout and readTotalTimeoutMultiplier
	//to 16rFFFFFFFF. Set readTotalTimeoutConstant to some reasonable timeout
	//interval (we've found we like 100 ms)
void settimeout(int to) {
 COMMTIMEOUTS cto={MAXDWORD,0,to,0,to};
 SetCommTimeouts(hSerialPort,&cto);
}

bool SerialConnect(int portnum,int baudrate) {
 if (hSerialPort!=INVALID_HANDLE_VALUE) return true;	// port already open?
 char portname[12];
 sprintf(portname,"\\\\.\\COM%d",portnum+1);
	// open the port
 hSerialPort=CreateFile(portname,GENERIC_READ|GENERIC_WRITE,0,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
 if (hSerialPort==INVALID_HANDLE_VALUE) {
  printf("Cannot open port %s!\n",portname);
  return false;
 }
 DCB dcb;
 dcb.DCBlength=sizeof(dcb);
 BOOL ok=GetCommState(hSerialPort,&dcb);
 if (ok) {
  dcb.BaudRate=baudrate;
  dcb.fParity=0;
  dcb.fOutxCtsFlow=1;
  dcb.fOutxDsrFlow=0;
  dcb.fDtrControl=DTR_CONTROL_ENABLE;
  dcb.fDsrSensitivity=0;
  dcb.fTXContinueOnXoff=0;
  dcb.fOutX=0;
  dcb.fInX=0;
  dcb.fNull=0;
  dcb.fRtsControl=RTS_CONTROL_HANDSHAKE;
  dcb.fAbortOnError=1;
  dcb.ByteSize=8;
  dcb.Parity=NOPARITY;
  dcb.StopBits=ONESTOPBIT;
		// configure the port
  ok=SetCommState(hSerialPort,&dcb);
 }
 if (!ok) {
  int err=GetLastError();
  CloseHandle(hSerialPort);
  hSerialPort=INVALID_HANDLE_VALUE;
  if (err) printf("Invalid port settings for %s!",portname);
  return false;
 }
 settimeout(250);
 SetupComm(hSerialPort,serialcommbuffer,serialcommbuffer);
 return true;
}

void SerialClose() {
 if (hSerialPort!=INVALID_HANDLE_VALUE)	{
  CloseHandle(hSerialPort);
  hSerialPort = INVALID_HANDLE_VALUE;
 }
}

bool Connected() {
 return hSerialPort!=INVALID_HANDLE_VALUE;
}

void ClearBuffers(DWORD purge) {
 PurgeComm(hSerialPort,purge);
}

Err TimedSendBytes(const char*sendbuf,DWORD numbytes,int32_t to_ms) {
 if (hSerialPort==INVALID_HANDLE_VALUE) return err_send;
 settimeout(to_ms);
 while (numbytes) {
  DWORD sent;
  if (!WriteFile(hSerialPort,sendbuf,numbytes,&sent,NULL)) {
   DWORD Errors;
   int err=GetLastError();
   ClearCommError(hSerialPort,&Errors,NULL);
   break;
  }
  sendbuf+=sent;
  numbytes-=sent;
 }
 if (numbytes) return err_send;
 return 0;
}

Err TimedReceiveBytes(char*recvbuf,DWORD&numbytes,int32_t to_ms) {
 if (hSerialPort==INVALID_HANDLE_VALUE) return err_recv;
 settimeout(to_ms);
 int32_t bytesneeded=numbytes;
 while (bytesneeded) {
  DWORD bytesreceived;
  if (!ReadFile(hSerialPort,recvbuf,bytesneeded,&bytesreceived,NULL)) {
   DWORD Errors;
   int err=GetLastError();
   ClearCommError(hSerialPort,&Errors,NULL);
  }
  bytesneeded-=bytesreceived;
  recvbuf+=bytesreceived;
 }
 numbytes-=bytesneeded;
 if (bytesneeded) return err_recv;
 return 0;
}

int TimedSendChar(char byte,int to_ms) {
 return TimedSendBytes(&byte,1,to_ms);
}

int TimedReceiveChar(char*ch,int32_t to_ms) {
 DWORD receive=1;
 return TimedReceiveBytes(ch,receive,to_ms);
}
